@@ -30,6 +30,7 @@ static char do_not_config;
 static unsigned int default_caps = CAP_DHCP | CAP_BOOTP | CAP_RARP;
 static int loop_timeout = -1;
 static int configured;
+static int bringup_first = 0;
 
 struct state {
 	int		state;
@@ -316,23 +317,30 @@ static int loop(void)
 	while (1) {
 		int timeout = 60;
 		int pending = 0;
+		int done = 0;
 		int timeout_ms;
 		int x;
 
 		for (s = slist; s; s = s->next) {
-			if (s->state == DEVST_COMPLETE)
+			DEBUG(("%s: state = %d\n", s->dev->name, s->state));
+
+			if (s->state == DEVST_COMPLETE) {
+				done++;
 				continue;
+			}
 
 			pending++;
 
-			if (s->expire - now.tv_sec <= 0)
+			if (s->expire - now.tv_sec <= 0) {
+				DEBUG(("timeout\n"));
 				process_timeout_event(s, now.tv_sec);
+			}
 
 			if (timeout > s->expire - now.tv_sec)
 				timeout = s->expire - now.tv_sec;
 		}
 
-		if (pending == 0)
+		if (pending == 0 || (bringup_first && done))
 			break;
 
 		timeout_ms = timeout * 1000;
@@ -447,59 +455,69 @@ static int parse_device(struct netdev *dev, const char *ip)
 {
 	char *cp;
 	int i, opt;
+	int is_ip = 0;
+
+	DEBUG(("IP-Config: parse_device: \"%s\"\n", ip));
 
 	if (strncmp(ip, "ip=", 3) == 0) {
 		ip += 3;
+		is_ip = 1;
 	}
 	else if (strncmp(ip, "nfsaddrs=", 9) == 0) {
 		ip += 9;
+		is_ip = 1;	/* Not sure about this...? */
 	}
 
-	if (strchr(ip, ':') == NULL) {
-		dev->name = ip;
-		goto done;
-	}
-
-	for (i = opt = 0; ip && *ip; ip = cp, opt++) {
-		if ((cp = strchr(ip, ':'))) {
-			*cp++ = '\0';
-		}
-		if (opt > 6) {
-			fprintf(stderr, "%s: too many options for %s\n",
-				progname, dev->name);
-			exit(1);
-		}
-
-		if (*ip == '\0')
-			continue;
-		DEBUG(("IP-Config: opt #%d: '%s'\n", opt, ip));
-		switch (opt) {
-		case 0:
-			parse_addr(&dev->ip_addr, ip);
-			dev->caps = 0;
-			break;
-		case 1:
-			parse_addr(&dev->ip_server, ip);
-			break;
-		case 2:
-			parse_addr(&dev->ip_gateway, ip);
-			break;
-		case 3:
-			parse_addr(&dev->ip_netmask, ip);
-			break;
-		case 4:
-			strncpy(dev->hostname, ip, SYS_NMLN - 1);
-			dev->hostname[SYS_NMLN - 1] = '\0';
-			break;
-		case 5:
-			dev->name = ip;
-			break;
-		case 6:
+	if ( !strchr(ip, ':') ) {
+		/* Only one option, e.g. "ip=dhcp", or it's an interface name */
+		if ( is_ip ) {
 			dev->caps = parse_proto(ip);
-			break;
+			bringup_first = 1;
+		} else {
+			dev->name = ip;
+		}
+	} else {
+		for (i = opt = 0; ip && *ip; ip = cp, opt++) {
+			if ((cp = strchr(ip, ':'))) {
+				*cp++ = '\0';
+			}
+			if (opt > 6) {
+				fprintf(stderr, "%s: too many options for %s\n",
+					progname, dev->name);
+				exit(1);
+			}
+			
+			if (*ip == '\0')
+				continue;
+			DEBUG(("IP-Config: opt #%d: '%s'\n", opt, ip));
+			switch (opt) {
+			case 0:
+				parse_addr(&dev->ip_addr, ip);
+				dev->caps = 0;
+				break;
+			case 1:
+				parse_addr(&dev->ip_server, ip);
+				break;
+			case 2:
+				parse_addr(&dev->ip_gateway, ip);
+				break;
+			case 3:
+				parse_addr(&dev->ip_netmask, ip);
+				break;
+			case 4:
+				strncpy(dev->hostname, ip, SYS_NMLN - 1);
+				dev->hostname[SYS_NMLN - 1] = '\0';
+				break;
+			case 5:
+				dev->name = ip;
+				break;
+			case 6:
+				dev->caps = parse_proto(ip);
+				break;
+			}
 		}
 	}
- done:
+
 	if (dev->name == NULL ||
 	    dev->name[0] == '\0' ||
 	    strcmp(dev->name, "all") == 0) {
@@ -617,6 +635,8 @@ static int add_all_devices(struct netdev *template)
 		if ( !(flags & IFF_LOOPBACK) &&
 		     (flags & (IFF_BROADCAST|IFF_POINTOPOINT)) )
 		{
+			DEBUG(("Trying to bring up %s\n", de->d_name));
+
 			if ( !(dev = add_device(de->d_name)) )
 				continue;
 			bringup_one_dev(template, dev);
@@ -663,7 +683,7 @@ int ipconfig_main(int argc, char *argv[])
 	srand48(now.tv_usec ^ (now.tv_sec << 24));
 
 	do {
-		c = getopt(argc, argv, "c:d:np:t:");
+		c = getopt(argc, argv, "c:d:onp:t:");
 		if (c == EOF)
 			break;
 
@@ -691,6 +711,9 @@ int ipconfig_main(int argc, char *argv[])
 				exit(1);
 			}
 			break;
+		case 'o':
+			bringup_first = 1;
+			break;
 		case 'n':
 			do_not_config = 1;
 			break;
